#include <stdio.h>
#include "sqlite3.h"
#include "kk_log.h"
#include "kk_hal.h"
#include "kk_dm_mng.h"
#include "kk_scene_pollution_check.h"
#include "kk_property_db.h"
kk_pmGrade_t g_pm25GradeMap[] = 
{
	{
		{
			{0,35,10},{36,75,1},{76,150,2},{151,250,3},{251,999,4}
		}
	},	
};
kk_pollution_t  g_pollutionMap[] = 
{
	{
		"CO2",
		{
			{0,1500,1},{1501,2500,2},{2501,3500,3},{3501,9999,4}
		}
	},
	{
		"PM2.5",
		{
			{0,75,1},{76,150,2},{151,250,3},{251,999,4}
		}
	},
	{
		"HCHO",
		{
			{0,40,1},{41,80,2},{81,150,3},{151,999,4}
		}
	},	
	{
		"TVOC",
		{
			{0,40,1},{41,80,2},{81,150,3},{151,999,4}
		}
	},
};
int kk_pollution_level_check(kk_pollution_level_t levelMap[],int val,int num)
{
	int i = 0;
	for(i = 0; i < num; i++){
		if(val > levelMap[i].min && val <= levelMap[i].max){
			return levelMap[i].level;
		}
	}
	return -1;
}
int kk_pollution_check(cJSON *param,int grade)
{
	int i = 0;
	int num = 0;
	int ret = 0;
	cJSON *item = NULL;
	if(param == NULL){
		return -1;
	}
	num = sizeof(g_pollutionMap)/sizeof(kk_pollution_t);
	for(i = 0; i < num; i++){
		item = cJSON_GetObjectItem(param,g_pollutionMap[i].identifier);
		if(item == NULL){
			continue;
		}
		if(item->type != cJSON_Number){
			continue;
		}
		ret = kk_pollution_level_check(g_pollutionMap[i].levelMap,item->valueint,4);
		if(ret != -1){
			if(grade == 1){
				if(ret == 1){
					continue;
				}else{
					return 0;
				}
			}else{
				if(grade == ret){
					return 1;
				}
			}	
		}
	}
	if(grade == 1 && ret == 1){
		return 1;
	}
	return 0;
}

int kk_pollution_condition_check(char *deviceCode,int grade)
{
    int num = 0;
    int i = 0;
    int ret = 0;
    char currentValue[64] = {0};
    num = sizeof(g_pollutionMap)/sizeof(kk_pollution_t);
    for(i = 0; i < num; i++){
        memset(currentValue,0x0,sizeof(currentValue));
        kk_property_db_get_value_directly(deviceCode,g_pollutionMap[i].identifier,currentValue);
        if(strcmp(currentValue,"") == 0){
            continue;
        }
		ret = kk_pollution_level_check(g_pollutionMap[i].levelMap,atoi(currentValue),4);
		if(ret != -1){
			if(grade == 1){
				if(ret == 1){
					continue;
				}else{
					return 0;
				}
			}else{
				if(grade == ret){
					return 1;
				}
			}	
		}        
    }
	if(grade == 1 && ret == 1){
		return 1;
	}
	return 0;
}

int kk_pm25_condition_check(char *deviceCode)
{
    int num = 0;
    int i = 0;
    int ret = 0;
    char currentValue[64] = {0};
    kk_property_db_get_value_directly(deviceCode,"PM2.5",currentValue);
    if(strcmp(currentValue,"") == 0){
        return 0;
    }
    return kk_pollution_level_check(g_pm25GradeMap[i].levelMap,atoi(currentValue),5);
}
static kk_TNBSceneId_list_t *p_TNBSceneId_list= NULL;
char *identifierSceneSpl[] = {
	(char*){"Temperature"},
	(char*){"Humidity"},
	(char*){"IlluminanceLevel"},
};
int kk_TNB_identifier_check(char *identifier)
{
	int num = 0;
	int i = 0;
	num = sizeof(identifierSceneSpl)/sizeof(char*);
	for(i = 0; i < num; i++){
		if(strcmp(identifierSceneSpl[i],identifier) == 0){
			INFO_PRINT("[%s]kk_TNB_identifier_check enter!!!\n",identifier);
			return 1;
		}
	}
	return 0;
}
int kk_TNB_value_check(char *identifier,char *deviceCode,char * compareType,char * compareValue)
{
	int res = 0;
	dm_mgr_dev_node_t *node = NULL;
	char currentValue[64] = {0};	
	res = dm_mgr_get_device_by_devicecode(deviceCode, &node);
	if(res != 0){
		return res;
	}	
	kk_property_db_get_value_directly(deviceCode,identifier,currentValue);
	if(!strcmp(compareType,"=") && strcmp(compareValue,currentValue) != 0){
		INFO_PRINT("currentValue[%s]compareValue[%s],kk_TNB_value_check enter!!!\n",currentValue,compareValue);
		res = 1;
	}else if(!strcmp(compareType,">") && strcmp(currentValue,compareValue) < 0){
		INFO_PRINT("currentValue[%s]compareValue[%s],kk_TNB_value_check enter!!!\n",currentValue,compareValue);
		res = 1;
	}else if(!strcmp(compareType,"<") && strcmp(currentValue,compareValue) > 0){
		INFO_PRINT("currentValue[%s]compareValue[%s],kk_TNB_value_check enter!!!\n",currentValue,compareValue);
		res = 1;
	}else{
		INFO_PRINT("currentValue[%s]compareValue[%s],compareType[%s]!!\n",currentValue,compareValue,compareType);
		res = -1;
	}
	return res;

}
int kk_find_TNBScene_execute(const char* sceneId)
{
	kk_TNBSceneId_list_t *ptr = p_TNBSceneId_list;
	while(ptr){
		if(strcmp(ptr->sceneId,sceneId) == 0){
			INFO_PRINT("[%s]kk_find_TNBScene_execute enter!!!\n",sceneId);
			return 1;
		}
		ptr = ptr->next;
	}
	return 0;
}
int kk_scene_push_TNBSceneId_list(const char* sceneId)
{
	if(sceneId == NULL){
		return INVALID_PARAMETER;
	}
	kk_TNBSceneId_list_t *ptr = NULL,*ptemp = NULL;
	ptemp = ptr = p_TNBSceneId_list;
	while(ptr){
		ptemp = ptr;
		ptr = ptr->next;
	}
	ptr = malloc(sizeof(kk_TNBSceneId_list_t));
	if(ptr == NULL) {
		return MEMORY_NOT_ENOUGH;
	}
	memset(ptr,0x0,sizeof(kk_TNBSceneId_list_t));
	memcpy(ptr->sceneId,sceneId,strlen(sceneId));
	if(p_TNBSceneId_list == NULL){
		p_TNBSceneId_list = ptr;
	}else{
		ptemp->next = ptr;
	}
	return SUCCESS_RETURN;
}
void kk_scene_rm_TNBSceneId_list(const char* sceneId)
{
	kk_TNBSceneId_list_t *ptr = p_TNBSceneId_list;
	kk_TNBSceneId_list_t *pTemp = NULL;
	while(ptr){
		if(strcmp(ptr->sceneId,sceneId) == 0){
			if(ptr == p_TNBSceneId_list){
				p_TNBSceneId_list = p_TNBSceneId_list->next;
				free(ptr);
				return;
			}
			else{
				pTemp->next = ptr->next;
				free(ptr);
				return;
			}	
		}
		pTemp = ptr;
		ptr = ptr->next;
	}
	return;
}
void kk_scene_print_TNBSceneId_list()
{
	kk_TNBSceneId_list_t *ptr = p_TNBSceneId_list;
	while(ptr){
		printf("kk_scene_print_TNBSceneId_list [%s]\n",ptr->sceneId);
		ptr = ptr->next;
	}
}